@@ -10,16 +10,18 @@ __all__ = [
     "get_data",
     "get_output_params",
     "get_return_value",
+    "o_execute_non_query",
+    "o_get_data",
 ]
 
 from types import TracebackType
-from typing import Any, List, Optional, Type, Union
+from typing import Any, List, Optional, Tuple, Type, Union
 
 import system.db
 from com.inductiveautomation.ignition.common import BasicDataset
 from java.lang import Thread
 
-from incendium.types import DictIntStrAny, SProcResult, String
+from incendium.helper.types import DictIntStringAny, SProcResult, String
 
 
 class DisposableConnection(object):
@@ -30,6 +32,9 @@ class DisposableConnection(object):
     resources.
     """
 
+    database = None  # type: String
+    retries = None  # type: int
+
     def __init__(self, database, retries=3):
         # type: (String, int) -> None
         """Disposable Connection initializer.
@@ -90,11 +95,11 @@ class DisposableConnection(object):
 
 
 class Param(object):
-    """Base class used for defining [In|Out]put parameters."""
+    """Base class used for defining [IN|OUT]PUT parameters."""
 
     def __init__(
         self,
-        name_or_index,  # type: Union[int, str]
+        name_or_index,  # type: Union[int, String]
         type_code,  # type: int
         value=None,  # type: Optional[Any]
     ):
@@ -110,9 +115,24 @@ class Param(object):
         self._type_code = type_code
         self._value = value
 
+    def __repr__(self):
+        """Compute the "official" string representation."""
+        return "{}(name_or_index={!r}, type_code={}, value={})".format(
+            self.__class__.__name__,
+            self.name_or_index,
+            self.type_code,
+            self.value,
+        )
+
+    def __str__(self):
+        """Compute the "informal" string representation."""
+        return "{!r}, {}, {}".format(
+            self.name_or_index, self.type_code, self.value
+        )
+
     @property
     def name_or_index(self):
-        # type: () -> Union[int, str]
+        # type: () -> Union[int, String]
         """Get value of name_or_index."""
         return self._name_or_index
 
@@ -133,7 +153,7 @@ class InParam(Param):
     """Class used for declaring INPUT parameters."""
 
     def __init__(self, name_or_index, type_code, value):
-        # type: (Union[int, str], int, Any) -> None
+        # type: (Union[int, String], int, Any) -> None
         """Create an instance of InParam.
 
         Args:
@@ -151,7 +171,7 @@ class OutParam(Param):
     """Class used for declaring OUTPUT parameters."""
 
     def __init__(self, name_or_index, type_code):
-        # type: (Union[int, str], int) -> None
+        # type: (Union[int, String], int) -> None
         """Create an instance of OutParam.
 
         Args:
@@ -241,7 +261,7 @@ def _execute_sp(
         "output_params": _out_params,
         "result_set": call.getResultSet() if get_result_set else None,
         "return_value": call.getReturnValue() if get_ret_val else None,
-        "update_count": call.getUpdateCount() if get_update_count else None,
+        "update_count": call.getUpdateCount() if get_update_count else -1,
     }
 
 
@@ -256,7 +276,7 @@ def check(stored_procedure, database="", params=None):
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        params: A Dictionary containing all parameters. Optional.
+        params: A Dictionary containing all INPUT parameters. Optional.
 
     Returns:
         The flag.
@@ -277,7 +297,7 @@ def execute_non_query(
     transaction=None,  # type: Optional[String]
     params=None,  # type: Optional[List[InParam]]
 ):
-    # type: (...) -> Optional[int]
+    # type: (...) -> int
     """Execute a stored procedure against the connection.
 
     Used for UPDATE, INSERT, and DELETE statements.
@@ -289,7 +309,8 @@ def execute_non_query(
             connection will be used. Optional.
         transaction: A transaction identifier. If omitted, the call will
             be executed in its own transaction. Optional.
-        params: A Dictionary containing all parameters. Optional.
+        params: A list containing all INPUT parameters as InParam
+            objects. Optional.
 
     Returns:
         The number of rows modified by the stored procedure, or -1 if
@@ -319,7 +340,8 @@ def get_data(
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
-        params: A Dictionary containing all parameters. Optional.
+        params: A list containing all INPUT parameters as InParam
+            objects. Optional.
 
     Returns:
         A Dataset that is the resulting data of the stored procedure
@@ -342,21 +364,23 @@ def get_output_params(
     transaction=None,  # type: Optional[String]
     params=None,  # type: Optional[List[InParam]]
 ):
-    # type: (...) -> DictIntStrAny
+    # type: (...) -> DictIntStringAny
     """Get the Output parameters from the Stored Procedure.
 
     Args:
         stored_procedure: The name of the stored procedure to execute.
-        output: A Dictionary containing all output parameters.
+        output: A list containing all OUTPUT parameters as OutParam
+            objects.
         database: The name of the database connection to execute
             against. If omitted or "", the project's default database
             connection will be used. Optional.
         transaction: A transaction identifier. If omitted, the call will
             be executed in its own transaction. Optional.
-        params: A Dictionary containing all parameters. Optional.
+        params: A list containing all INPUT parameters as InParam
+            objects. Optional.
 
     Returns:
-        Result's output_params.
+        A Python dictionary of OUTPUT paramaters.
     """
     result = _execute_sp(
         stored_procedure,
@@ -388,7 +412,8 @@ def get_return_value(
             connection will be used. Optional.
         transaction: A transaction identifier. If omitted, the call will
             be executed in its own transaction. Optional.
-        params: A Dictionary containing all parameters. Optional.
+        params: A list containing all INPUT parameters as InParam
+            objects. Optional.
 
     Returns:
         The return value.
@@ -403,3 +428,78 @@ def get_return_value(
     )
 
     return result["return_value"]
+
+
+def o_execute_non_query(
+    stored_procedure,  # type: String
+    out_params,  # type: List[OutParam]
+    database="",  # type: String
+    transaction=None,  # type: Optional[String]
+    in_params=None,  # type: Optional[List[InParam]]
+):
+    # type: (...) -> Tuple[int, DictIntStringAny]
+    """Execute a stored procedure against the connection.
+
+    Used for UPDATE, INSERT, and DELETE statements which return OUTPUT
+    parameters.
+
+    Args:
+        stored_procedure: The name of the stored procedure to execute.
+        out_params: A Dictionary containing all OUTPUT parameters.
+        database: The name of the database connection to execute
+            against. If omitted or "", the project's default database
+            connection will be used. Optional.
+        transaction: A transaction identifier. If omitted, the call will
+            be executed in its own transaction. Optional.
+        in_params: A Dictionary containing all INPUT parameters.
+            Optional.
+
+    Returns:
+        A tuple containing the number of rows modified by the stored
+        procedure, or -1 if not applicable, and the OUTPUT parameters.
+    """
+    result = _execute_sp(
+        stored_procedure,
+        database=database,
+        transaction=transaction,
+        in_params=in_params,
+        out_params=out_params,
+        get_out_params=True,
+        get_update_count=True,
+    )
+
+    return result["update_count"], result["output_params"]
+
+
+def o_get_data(
+    stored_procedure,  # type: String
+    out_params,  # type: List[OutParam]
+    database="",  # type: String
+    in_params=None,  # type: Optional[List[InParam]]
+):
+    # type: (...) -> Tuple[Optional[BasicDataset], DictIntStringAny]
+    """Get data by executing a stored procedure and OUTPUT parameters.
+
+    Args:
+        stored_procedure: The name of the stored procedure to execute.
+        out_params: A Dictionary containing all OUTPUT parameters.
+        database: The name of the database connection to execute
+            against. If omitted or "", the project's default database
+            connection will be used. Optional.
+        in_params: A Dictionary containing all INPUT parameters.
+            Optional.
+
+    Returns:
+        A tuple containing a Dataset that is the resulting data of the
+        stored procedure call, if any, and the OUTPUT parameters.
+    """
+    result = _execute_sp(
+        stored_procedure,
+        database=database,
+        in_params=in_params,
+        out_params=out_params,
+        get_out_params=True,
+        get_result_set=True,
+    )
+
+    return result["result_set"], result["output_params"]
